﻿Imports System.Drawing.Imaging
Imports System.IO

Public Class Form1

  Dim NullTile As TileClass
  Dim DragTile As TileClass
  Dim Tiles As New Collection
  Dim Mode As String
  Shadows Scale As String
  Shared CellSize As Integer
  Shared ScrWidth As Integer
  Shared ScrHeight As Integer
  Shared PatioWidth As Integer
  Shared PatioHeight As Integer
  Shared Gutter As Integer = 4
  Dim AbortInterrupFlag As Boolean = True
  Dim EdgePen As New Pen(Color.Black, 8)

  Public Class Collection
    Inherits CollectionBase
    Public Sub Add(t As TileClass)
      List.Add(t)
    End Sub
    Public Sub Remove(t As TileClass)
      List.Remove(t)
    End Sub
    Public Sub Empty()
      List.Clear()
    End Sub
    Public Function Index(t As TileClass)
      Return List.IndexOf(t)
    End Function
  End Class

  Public Class TileClass
    Public x, y, w, h As Integer
    Public type As String
    Public colour As Color
    Public used As Boolean

    Public Sub New(xx As Integer, yy As Integer, t As String)
      x = xx * CellSize + Gutter / 2
      y = yy * CellSize + Gutter / 2
      type = t
      used = False
      Select Case t
        Case "1mer" : colour = Color.Yellow : w = 1 : h = 1
        Case "H2mer" : colour = Color.Green : w = 2 : h = 1
        Case "V2mer" : colour = Color.Green : w = 1 : h = 2
        Case "4mer" : colour = Color.Blue : w = 2 : h = 2
        Case "H6mer" : colour = Color.Red : w = 3 : h = 2
        Case "V6mer" : colour = Color.Red : w = 2 : h = 3
      End Select
      w = w * CellSize - Gutter
      h = h * CellSize - Gutter
    End Sub
  End Class

  Private Sub Form1_Load(sender As Object, e As System.EventArgs) Handles Me.Load
    PictureBox1.AllowDrop = True
    NumericUpDown1.Value = 1
    NumericUpDown2.Value = 2
    NumericUpDown3.Value = 2
    NumericUpDown4.Value = 1
    NumericUpDown5.Value = 5
    NumericUpDown6.Value = 4
    NumericUpDown1.Enabled = False
    RadioButton1.Checked = True
    RadioButton3.Checked = True
    CheckBox1.Checked = True
    AbortInterrupFlag = False
    NullTile = New TileClass(0, 0, "Null")
    CalculateArea()
    SetScale()
    NewSet()
  End Sub

  Private Sub SetScale()
    If Scale = "Large" Then CellSize = 60 Else CellSize = 30
    ScrWidth = 900 \ CellSize
    ScrHeight = 600 \ CellSize
    NumericUpDown5.Maximum = ScrWidth
    NumericUpDown6.Maximum = ScrHeight
  End Sub

  Private Sub NewSet()
    Tiles.Empty()
    Dim N1 As Integer = NumericUpDown1.Value
    Dim N2 As Integer = NumericUpDown2.Value
    Dim N4 As Integer = NumericUpDown3.Value
    Dim N6 As Integer = NumericUpDown4.Value
    If Mode = "Free" Then
      N1 = 12 : N2 = 18 : N4 = 4 : N6 = 2
    End If
    Dim n As Integer = N6
    Dim x As Integer = 1
    Dim y As Integer = 8
    If Scale = "Medium" Then y = 18
    n = N6
    While n > 0 : Tiles.Add(New TileClass(x, y, "H6mer"))
      n -= 1
    End While
    n = N4
    x = 5
    While n > 0
      Tiles.Add(New TileClass(x, y, "4mer")) : n -= 1
      x += 2
      If x > ScrWidth - 3 Then x = 8
    End While
    n = N2
    x = 1 : y = 1
    While n > 0
      Tiles.Add(New TileClass(x, y, "H2mer"))
      x += 2 : n -= 1
      If x > ScrWidth - 3 Then x = 1
    End While
    n = N1
    x = 1 : y = 0
    While n > 0
      Tiles.Add(New TileClass(x, y, "1mer")) : n -= 1
      x += 1
      If x > ScrWidth - 3 Then x = 1
    End While
    PictureBox1.Invalidate()
  End Sub

  Private Sub PictureBox1_MouseClick(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseClick
    Dim t As TileClass = FindTile(e.X, e.Y)
    Dim s As TileClass
    Dim i As Integer = t.x \ CellSize
    Dim j As Integer = t.y \ CellSize
    If e.Button = Windows.Forms.MouseButtons.Middle Then
      ' join tiles
      Select Case t.type
        Case "4mer"
          Select Case True
            Case TestEdge(i, j, i + 2, j)
              ' top edge
              s = FindTile2(i, j - 1)
              If s.type = "H2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j - 1, "V6mer"))
              End If
            Case TestEdge(i + 2, j, i + 2, j + 2)
              ' right edge
              s = FindTile2(i + 2, j)
              If s.type = "V2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "H6mer"))
              End If
            Case TestEdge(i, j + 2, i + 2, j + 2)
              ' bottom edge
              s = FindTile2(i, j + 2)
              If s.type = "H2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "V6mer"))
              End If
            Case TestEdge(i, j, i, j + 2)
              ' left edge
              s = FindTile2(i - 1, j)
              If s.type = "V2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i - 1, j, "H6mer"))
              End If
          End Select
        Case "H2mer"
          Select Case True
            Case TestEdge(i, j, i + 2, j)
              ' top edge
              s = FindTile2(i, j - 1)
              If s.type = "H2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j - 1, "4mer"))
              End If
            Case TestEdge(i, j + 1, i + 2, j + 1)
              ' bottom edge
              s = FindTile2(i, j + 1)
              If s.type = "H2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "4mer"))
              End If
          End Select
        Case "V2mer"
          Select Case True
            Case TestEdge(i + 1, j, i + 1, j + 2)
              ' right edge
              s = FindTile2(i + 1, j)
              If s.type = "V2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "4mer"))
              End If
            Case TestEdge(i, j, i, j + 2)
              ' left edge
              s = FindTile2(i - 1, j)
              If s.type = "V2mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i - 1, j, "4mer"))
              End If
          End Select
        Case "1mer"
          Select Case True
            Case TestEdge(i, j, i + 1, j)
              ' top edge
              s = FindTile2(i, j - 1)
              If s.type = "1mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j - 1, "V2mer"))
              End If
            Case TestEdge(i + 1, j, i + 1, j + 1)
              ' right edge
              s = FindTile2(i + 1, j)
              If s.type = "1mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "H2mer"))
              End If
            Case TestEdge(i, j + 1, i + 1, j + 1)
              ' bottom edge
              s = FindTile2(i, j + 1)
              If s.type = "1mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i, j, "V2mer"))
              End If
            Case TestEdge(i, j, i, j + 1)
              ' left edge
              s = FindTile2(i - 1, j)
              If s.type = "1mer" Then
                Tiles.RemoveAt(Tiles.Index(t))
                Tiles.RemoveAt(Tiles.Index(s))
                Tiles.Add(New TileClass(i - 1, j, "H2mer"))
              End If
          End Select
      End Select
    ElseIf e.Button = Windows.Forms.MouseButtons.Right Then
      ' split tile
      Select Case t.type
        Case "H6mer"
          Tiles.Add(New TileClass(i, j, "4mer"))
          Tiles.Add(New TileClass(i + 2, j, "V2mer"))
        Case "V6mer"
          Tiles.Add(New TileClass(i, j, "4mer"))
          Tiles.Add(New TileClass(i, j + 2, "H2mer"))
        Case "4mer"
          Tiles.Add(New TileClass(i, j, "H2mer"))
          Tiles.Add(New TileClass(i, j + 1, "H2mer"))
        Case "H2mer"
          Tiles.Add(New TileClass(i, j, "1mer"))
          Tiles.Add(New TileClass(i + 1, j, "1mer"))
        Case "V2mer"
          Tiles.Add(New TileClass(i, j, "1mer"))
          Tiles.Add(New TileClass(i, j + 1, "1mer"))
      End Select
      If t.type <> "1mer" Then Tiles.RemoveAt(Tiles.Index(t))
    End If
  End Sub

  Private Sub PictureBox1_MouseDoubleClick(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseDoubleClick
    Dim t As TileClass = FindTile(e.X, e.Y)
    If My.Computer.Keyboard.ShiftKeyDown Then
      Tiles.Remove(t)
    Else
      Dim a As Integer
      Select Case t.type
        Case "H2mer" : t.type = "V2mer" : a = t.w : t.w = t.h : t.h = a
        Case "V2mer" : t.type = "H2mer" : a = t.w : t.w = t.h : t.h = a
        Case "H6mer" : t.type = "V6mer" : a = t.w : t.w = t.h : t.h = a
        Case "V6mer" : t.type = "H6mer" : a = t.w : t.w = t.h : t.h = a
      End Select
    End If
    PictureBox1.Invalidate()
  End Sub

  Private Sub PictureBox1_MouseDown(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseDown
    DragTile = FindTile(e.X, e.Y)
    If RadioButton1.Checked Then
      Select Case DragTile.type
        Case "1mer" : Me.Cursor = New Cursor(My.Resources.H1mer.Handle)
        Case "H2mer" : Me.Cursor = New Cursor(My.Resources.h2mer.Handle)
        Case "V2mer" : Me.Cursor = New Cursor(My.Resources.v2mer.Handle)
        Case "4mer" : Me.Cursor = New Cursor(My.Resources.H4mer.Handle)
        Case "H6mer" : Me.Cursor = New Cursor(My.Resources.H6mer.Handle)
        Case "V6mer" : Me.Cursor = New Cursor(My.Resources.V6mer.Handle)
      End Select
    Else
      Select Case DragTile.type
        Case "1mer" : Me.Cursor = New Cursor(My.Resources.SH1mer.Handle)
        Case "H2mer" : Me.Cursor = New Cursor(My.Resources.SH2mer.Handle)
        Case "V2mer" : Me.Cursor = New Cursor(My.Resources.SV2mer.Handle)
        Case "4mer" : Me.Cursor = New Cursor(My.Resources.SH4mer.Handle)
        Case "H6mer" : Me.Cursor = New Cursor(My.Resources.SH6mer.Handle)
        Case "V6mer" : Me.Cursor = New Cursor(My.Resources.SV6mer.Handle)
      End Select
    End If
  End Sub

  Private Sub PictureBox1_MouseMove(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseMove
  End Sub

  Private Sub PictureBox1_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseUp
    DragTile.x = ((e.X - DragTile.w \ 2 + CellSize \ 2) \ CellSize) * CellSize + Gutter / 2
    DragTile.y = ((e.Y - DragTile.h \ 2 + CellSize \ 2) \ CellSize) * CellSize + Gutter / 2
    DragTile.used = True
    Me.Cursor = Cursors.Default
    PictureBox1.Invalidate()
  End Sub

  Private Function FindTile(x As Integer, y As Integer) As TileClass
    ' finds the last tile in the collection
    Dim s As TileClass = NullTile
    For Each t In Tiles
      If x > t.x And x < t.x + t.w And y > t.y And y < t.y + t.h Then s = t
    Next
    Return s
  End Function

  Private Function FindTile2(i As Integer, j As Integer) As TileClass
    Dim s As TileClass = NullTile
    For Each t In Tiles
      If i = t.x \ CellSize And j = t.y \ CellSize Then s = t
    Next
    Return s
  End Function

  Private Function MinX()
    Dim a As Integer = 1000
    For Each t In Tiles
      If t.x < a Then a = t.x
    Next
    Return a
  End Function

  Private Function MinY()
    Dim a As Integer = 1000
    For Each t In Tiles
      If t.y < a Then a = t.y
    Next
    Return a
  End Function

  Private Function MaxX()
    Dim a As Integer = 0
    For Each t In Tiles
      If t.x + t.w > a Then a = t.x + t.w
    Next
    Return a + 1
  End Function

  Private Function MaxY()
    Dim a As Integer = 0
    For Each t In Tiles
      If t.y + t.h > a Then a = t.y + t.h
    Next
    Return a + 1
  End Function

  Private Sub PictureBox1_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles PictureBox1.Paint
    Dim a As Integer = (ScrWidth - NumericUpDown5.Value) \ 2
    Dim b As Integer = (ScrHeight - NumericUpDown6.Value) \ 2
    If Mode = "Fixed" Then
      e.Graphics.FillRectangle(New SolidBrush(Color.Gainsboro), a * CellSize, b * CellSize, NumericUpDown5.Value * CellSize, NumericUpDown6.Value * CellSize)
    End If
    'e.Graphics.DrawRectangle(Pens.Blue, MinX(), MinY(), MaxX() - MinX(), MaxY() - MinY())
    For Each Tile In Tiles
      DrawTile(e.Graphics, Tile)
    Next
    If CheckBox1.Checked Then
      For i As Integer = 1 To ScrWidth - 1
        If TestVline(i) = False Then e.Graphics.DrawLine(Pens.Red, i * CellSize, 0, i * CellSize, ScrHeight * CellSize)
      Next
      For i As Integer = 1 To ScrHeight - 1
        If TestHline(i) = False Then e.Graphics.DrawLine(Pens.Red, 0, i * CellSize, ScrWidth * CellSize, i * CellSize)
      Next
      For i As Integer = 0 To ScrWidth - 1
        For j As Integer = 0 To ScrHeight - 1
          If TestEdge(i, j, i + 1, j) Then DrawEdge(e.Graphics, i, j, i + 1, j)
          If TestEdge(i, j, i + 2, j) Then DrawEdge(e.Graphics, i, j, i + 2, j)
          If TestEdge(i, j, i + 3, j) Then DrawEdge(e.Graphics, i, j, i + 3, j)
          If TestEdge(i, j, i, j + 1) Then DrawEdge(e.Graphics, i, j, i, j + 1)
          If TestEdge(i, j, i, j + 2) Then DrawEdge(e.Graphics, i, j, i, j + 2)
          If TestEdge(i, j, i, j + 3) Then DrawEdge(e.Graphics, i, j, i, j + 3)
        Next
      Next
      For i As Integer = 1 To ScrWidth - 1
        For j As Integer = 1 To ScrHeight - 1
          If TestVetex(i, j) Then DrawSpot(e.Graphics, i, j)
        Next
      Next
    End If
  End Sub

  Private Function TestVetex(i As Integer, j As Integer) As Boolean
    Dim n As Integer = 0
    For Each Tile In Tiles
      If BelongTo(i, j, Tile) And Tile.used Then n += 1
    Next
    Return n > 2
  End Function

  Private Function TestEdge(i As Integer, j As Integer, k As Integer, l As Integer) As Boolean
    If k > ScrWidth Or l > ScrHeight Then Return False
    Dim n As Integer = 0 ' number of tiles which share an edge
    Dim m As Integer = 0 ' type of tiles (bit 0 = monomer bit 1 = dimer)
    For Each Tile In Tiles
      If BelongTo(i, j, Tile) AndAlso BelongTo(k, l, Tile) AndAlso Tile.used Then
        n += 1
        If Tile.Type = "1mer" Then m = m Or 1
        If Tile.Type = "H2mer" Or Tile.Type = "V2mer" Then m = m Or 2
      End If
    Next
    If m = 3 Then Return False ' monomers & dimer
    Return n = 2
  End Function

  Private Function TestVline(i As Integer) As Boolean
    Dim a As Integer = (ScrWidth - PatioWidth) \ 2
    If Mode = "Fixed" And (i <= a Or i >= a + PatioWidth) Then Return True
    For Each tile In Tiles
      If VCrosses(i, tile) Then Return True
    Next
    Return False
  End Function

  Private Function TestHline(j As Integer) As Boolean
    Dim b As Integer = (ScrHeight - PatioHeight) \ 2
    If Mode = "Fixed" And (j <= b Or j >= b + PatioHeight) Then Return True
    For Each tile In Tiles
      If HCrosses(j, tile) Then Return True
    Next
    Return False
  End Function

  Private Function VCrosses(i As Integer, t As TileClass) As Boolean
    If t.type = "1mer" Or Not t.used Then Return False
    Return i > Snap(t.x) And i < Snap(t.x + t.w)
  End Function

  Private Function HCrosses(j As Integer, t As TileClass) As Boolean
    If t.type = "1mer" Or Not t.used Then Return False
    Return j > Snap(t.y) And j < Snap(t.y + t.h)
  End Function

  Private Function BelongTo(i As Integer, j As Integer, Tile As TileClass) As Boolean
    If IsSameVertex(i, j, Tile.x, Tile.y) Then Return True
    If IsSameVertex(i, j, Tile.x + Tile.w, Tile.y) Then Return True
    If IsSameVertex(i, j, Tile.x, Tile.y + Tile.h) Then Return True
    If IsSameVertex(i, j, Tile.x + Tile.w, Tile.y + Tile.h) Then Return True
    Return False
  End Function

  Private Function IsSameVertex(i As Integer, j As Integer, x As Integer, y As Integer) As Boolean
    Return Snap(x) = i And Snap(y) = j
  End Function

  Private Function Snap(x As Integer) As Integer
    Return (x + CellSize \ 2) \ CellSize
  End Function

  Private Sub DrawTile(g As Graphics, t As TileClass)
    g.FillRectangle(New SolidBrush(t.colour), t.x, t.y, t.w, t.h)
    g.DrawRectangle(Pens.Black, t.x, t.y, t.w, t.h)
  End Sub

  Private Sub DrawSpot(g As Graphics, i As Integer, j As Integer)
    g.FillEllipse(Brushes.Black, i * CellSize - 8, j * CellSize - 8, 16, 16)
    g.FillEllipse(Brushes.White, i * CellSize - 4, j * CellSize - 4, 8, 8)
  End Sub

  Private Sub DrawEdge(g As Graphics, i As Integer, j As Integer, k As Integer, l As Integer)
    g.DrawLine(EdgePen, i * CellSize, j * CellSize, k * CellSize, l * CellSize)
  End Sub

  Private Sub Button1_Click(sender As System.Object, e As System.EventArgs) Handles Button1.Click
    NewSet()
  End Sub

  Private Sub CalculateArea()
    Dim Area = PatioWidth * PatioHeight
    Dim Total As Integer = 2 * NumericUpDown2.Value + 4 * NumericUpDown3.Value + 6 * NumericUpDown4.Value
    If Total > Area Then
      NumericUpDown1.Value = 0
    ElseIf Area - Total > 50 Then
      NumericUpDown1.Value = 50
    Else
      NumericUpDown1.Value = Area - Total
    End If
  End Sub

  Private Sub MersChanged(sender As System.Object, e As System.EventArgs) Handles NumericUpDown2.ValueChanged, NumericUpDown3.ValueChanged, NumericUpDown4.ValueChanged
    CalculateArea()
  End Sub

  Private Sub NumericUpDown5_ValueChanged(sender As System.Object, e As System.EventArgs) Handles NumericUpDown5.ValueChanged, NumericUpDown6.ValueChanged
    PatioWidth = NumericUpDown5.Value : PatioHeight = NumericUpDown6.Value
    CalculateArea()
  End Sub

  Private Sub Button2_Click(sender As System.Object, e As System.EventArgs) Handles Button2.Click
    ' save image
    Dim a As Integer = MinX()
    Dim b As Integer = MinY()
    Dim w As Integer = MaxX() - MinX()
    Dim h As Integer = MaxY() - MinY()
    Dim bm As New Bitmap(w, h)
    Dim filename As String
    Dim bmg As Graphics = Graphics.FromImage(bm)
    bmg.Clear(Color.WhiteSmoke)
    For Each t In Tiles
      bmg.FillRectangle(New SolidBrush(t.colour), t.x - a, t.y - b, t.w, t.h)
      bmg.DrawRectangle(Pens.Black, t.x - a, t.y - b, t.w, t.h)
    Next
    If SaveFileDialog1.ShowDialog = DialogResult.OK Then
      filename = SaveFileDialog1.FileName
      If Not filename.EndsWith(".gif") Then filename &= ".gif"
      bm.Save(filename, ImageFormat.Gif)
    End If
  End Sub

  Private Sub CheckBox1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles CheckBox1.CheckedChanged
    PictureBox1.Invalidate()
  End Sub

  Private Sub RadioButton1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles RadioButton1.CheckedChanged
    If RadioButton1.Checked Then
      Scale = "Large"
    Else
      Scale = "Medium"
    End If
    SetScale()
    If AbortInterrupFlag Then Exit Sub
    NewSet()
  End Sub

  Private Sub RadioButton3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles RadioButton3.CheckedChanged
    If RadioButton3.Checked Then
      Mode = "Fixed"
      GroupBox1.BringToFront()
    Else
      Mode = "Free"
      GroupBox3.BringToFront()
    End If
    If AbortInterrupFlag Then Exit Sub
    NewSet()
  End Sub

  Private Sub Button3_Click(sender As System.Object, e As System.EventArgs) Handles Button3.Click
    For i As Integer = 1 To 12
      Tiles.Add(New TileClass(i, 0, "1mer"))
    Next
    PictureBox1.Invalidate()
  End Sub

  Private Sub Button4_Click(sender As System.Object, e As System.EventArgs) Handles Button4.Click
    For i As Integer = 1 To 11 Step 2
      Tiles.Add(New TileClass(i, 1, "H2mer"))
    Next
    PictureBox1.Invalidate()
  End Sub

  Private Sub Button5_Click(sender As System.Object, e As System.EventArgs) Handles Button5.Click
    Dim y As Integer = 8
    If Scale = "Medium" Then y = 18
    For i As Integer = 5 To 11 Step 2
      Tiles.Add(New TileClass(i, y, "4mer"))
    Next
    PictureBox1.Invalidate()
  End Sub

  Private Sub Button6_Click(sender As System.Object, e As System.EventArgs) Handles Button6.Click
    Dim y As Integer = 8
    If Scale = "Medium" Then y = 18
    For i As Integer = 1 To 4
      Tiles.Add(New TileClass(1, y, "H6mer"))
    Next
    PictureBox1.Invalidate()
  End Sub

  Private Sub Button8_Click(sender As System.Object, e As System.EventArgs) Handles Button8.Click
    ' remove unused tiles
    Dim y As Integer = 8
    If Scale = "Medium" Then y = 18
    Dim f As Boolean = True
    While f
      f = False
      For Each tile In Tiles
        If tile.used = False Then Tiles.Remove(tile) : f = True : Exit For
      Next
    End While
    PictureBox1.Invalidate()
  End Sub

  Private Sub Button9_Click(sender As System.Object, e As System.EventArgs) Handles Button9.Click
    'start again
    NewSet()
  End Sub

  Private Sub Button7_Click(sender As System.Object, e As System.EventArgs) Handles Button7.Click
    'Save Solution as text file
    Dim filename As String
    If SaveFileDialog2.ShowDialog = DialogResult.OK Then
      filename = SaveFileDialog2.FileName
      If Not filename.EndsWith(".txt") Then filename &= ".txt"
      Dim filestream As New FileStream(filename, FileMode.Create, FileAccess.Write)
      Dim streamwriter As New StreamWriter(filestream)
      Try
        streamwriter.WriteLine("Tatami Tiler")
        streamwriter.WriteLine(Mode)
        streamwriter.WriteLine(Scale)
        streamwriter.WriteLine(Str(PatioWidth) & "," & Str(PatioHeight))
        For Each tile In Tiles
          streamwriter.WriteLine(Str(tile.x \ CellSize) & "," & Str(tile.y \ CellSize) & "," & tile.Type)
        Next
        streamwriter.Close()
      Catch ex As IOException
        MsgBox(ex.Message)
      End Try
    End If
  End Sub

  Private Sub PictureBox1_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles PictureBox1.DragEnter
    Beep()
    If e.Data.GetDataPresent(DataFormats.FileDrop) Then
      e.Effect = DragDropEffects.Copy
    Else
      e.Effect = DragDropEffects.None
    End If
  End Sub

  Private Sub PictureBox1_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles PictureBox1.DragDrop
    Dim filenames As String()
    filenames = DirectCast(e.Data.GetData(DataFormats.FileDrop), String())
    LoadFile(filenames(0))
  End Sub

  Private Sub LoadFile(filename)
    Dim filestream As New FileStream(filename, FileMode.Open, FileAccess.Read)
    Dim streamreader As New StreamReader(filestream)
    Try
      Text = streamreader.ReadLine
      If Text <> "Tatami Tiler" Then Throw New System.IO.IOException("This is not a Tatami Tiler file.")
      Tiles.Clear()
      Mode = streamreader.ReadLine
      If Mode = "Fixed" Then RadioButton3.Checked = True Else RadioButton4.Checked = True
      Scale = streamreader.ReadLine
      If Scale = "Large" Then RadioButton1.Checked = True Else RadioButton2.Checked = True
      SetScale()
      Text = streamreader.ReadLine
      PatioWidth = Val(Text) : Text = Text.Substring(InStr(Text, ","))
      PatioHeight = Val(Text)
      NumericUpDown5.Value = PatioWidth : NumericUpDown6.Value = PatioHeight
      ' Debug.Print(Mode & " " & Scale & " " & Str(PatioWidth) & Str(PatioHeight))
      Dim i, j As Integer
      While Not streamreader.EndOfStream
        Text = streamreader.ReadLine
        i = Val(Text) : Text = Text.Substring(InStr(Text, ","))
        j = Val(Text) : Text = Text.Substring(InStr(Text, ","))
        'Debug.Print(Str(i) & Str(j) & Text)
        Tiles.Add(New TileClass(i, j, Text))
      End While
      streamreader.Close()
      PictureBox1.Invalidate()
    Catch ex As IOException
      MsgBox(ex.Message)
    End Try
  End Sub
End Class
